#!/usr/bin/perl
###############################################################################
#
#	Konvertiert WAV-Dateien fr den AC1 (Monitor 3.1) ins binre Z80-Format 
#
#	06.08.2018	by Volker Pohlers created
#
###############################################################################

use strict;
use Fcntl ":seek";

# $ARGV[0] = 'd:\\hobby3\\ac1\\wav2z80\\mon_v31_16.wav';

die <<"help" unless @ARGV;
konvertiert WAV-Dateien mit AC1-Programm in Z80-Datei
Wave-Datei: 8 Bit, 22 kHz, Mono

- Es wird nicht kontrolliert, ob die WAV-Datei den Vorgaben entspricht
- Der Dateiname aus Headersave-Header wird fr z80 genutzt

help

our $bufsize = 8*2**10;	# 8k
our $pos = $bufsize;
our $buf;
our $offset;
our $abort;

open IN, $ARGV[0] or die $!;
binmode IN;
#seek IN, 54, SEEK_CUR;	# Wave-Header berspringen
seek IN, 60, SEEK_CUR;	# Wave-Header berspringen

# print "Pos ", tell (IN), "\n"; 

#########################################
# WAV lesen
#########################################

# ein Byte lesen
sub ReadWav {
	my $b;
	if ($pos >= $bufsize) {
		$pos = $pos-$bufsize;
		$bufsize = read(IN, $buf, $bufsize);
		$abort = 1 if $bufsize == 0;
		print "ABORT!\n" if $abort;
	}
	$b = ord(substr($buf, $pos++, 1));
#print "Byte ", tell (IN)-$bufsize+$pos, ' ', $b, "\n"; 
	return $b;
}

#########################################
# AC1-Format Lese-Routinen 
#########################################

#Samples
#15 = 6500
#7 = 13987
#8 = 13987

# 1 Bit = 1 Schwingung 1500Hz = 666s lang,
# 0 entspricht |_ und 1 entspricht _|
 
# Vorton 512 x 00-Bytes
# Syncbyte E6
# 'U' + Name 6 Byte
# 256 x 00-Bytes
# Blocks '<' + Blocklnge + max. 256 Byte Daten + Prfsumme
# Programmende 'x' + sadr




# Flankenwechsel suchen
sub ReadBit {
#print "ReadBit ", tell (IN)-$bufsize+$pos, ' ', $lastb, "\n"; 
	my $w;
	if (ReadWav() < $offset){ do { } until ((ReadWav() >= $offset) or $abort); $w = 1 }
	else			{ do { } until ((ReadWav() <= $offset) or $abort); $w = 0 }

	$pos += 10;	# 2/3 * (7+8)
	return $w;
}

my $syncbyte;
sub ReadByte {
	my $i, my $b;
	$b=0;

	for( $i=0;$i<=7;$i++) {
	    $b = $b * 2 + ReadBit();
#print "Bytes ", tell (IN)+$pos, "\n"; 
#print $w, ' ';
#printf ("bits %d %.2x\n", $i, $b);
	}
	$b ^= $syncbyte;
	
	#printf "ReadByte %.2X\n", $b;
	return $b;
}

#########################################
# Main
#########################################


# Mittelwert bestimmen (Gleichspannungsanteil)
# JKCEmu produziert Werte 0x80 und 0x5C

{
	my $min = 0xff;
	my $max = 0;
	for (1..1000) {
		my $b = ReadWav();
		$min = $b if $min > $b;
		$max = $b if $max < $b;
	}
	$offset = int(($max+$min)/2);
	print "DC-Offset $offset\n";
	# Puffer rcksetzen
	$pos = 0;
}

neues_file:

our @tapbuf = ();

# Vorton
my $b=0;
while ( !$abort ) {
        $b = $b * 2 + ReadBit();
        $b &= 0xFF;
       # printf "%.2X", $b;

        # Synchronisationsmuster
        if ($b == 0xE6) { $syncbyte = 0; last }
        if ($b == 0x19) { $syncbyte = 0xFF; last } 
}
printf "Syncbyte %.2X\n", $b;

# Name (Kennbyte 'U')
my $filename = '';
while ( ReadByte() != 0x55 and not $abort ) {}	# Kennbyte suchen
for (1..16) { $filename .= chr (ReadByte()) }	# max 16 (6) Zeichen
printf "Filename: %s\n", $filename;

# Hauptschleife
my $aadr = 0xFFFF;
my $badr = 0;
my $eadr = 0;
my $sadr = 0;

while ( !$abort ) {
	$b = ReadByte();
	
	# Ende
	if ($b == 0x78) {
		# 2 Byte startadr lesen
		$sadr = ReadByte()+ReadByte()*256;
		printf "startadr: %.4X\n", $sadr;			
		last; }
	# block
	elsif ($b == 0x3c) { 
		my $checksum = 0;
		my $anzahl = ReadByte(); # $checksum += $anzahl;
		if ($anzahl == 0) { $anzahl = 256 };
		$badr = ReadByte(); 	$checksum += $badr;
		$b = ReadByte();	$checksum += $b;
		$badr = $badr + $b*256;
		$eadr = $badr + $anzahl-1;
		if ($aadr > $badr) {$aadr = $badr}
		printf "block: %.4X - %4X\n", $badr, $eadr;			
		for (1..$anzahl) {
			$b = ReadByte(); $checksum += $b;
			#printf "%.2X", $b;
			push (@tapbuf, $b);
		}	
		$b = ReadByte();
		if (($checksum & 0xff) != $b) {
			print "Prfsummenfehler! ",$checksum & 0xff," $b\n";
		}
	#} else { # Pause nach Name
	#	printf "\n BYTE %.2X", $b;
	}
}


#########################################
# Z80-Datei schreiben
#########################################


$filename =~ s/[\x00 ]+$//;	# Lerrzeichen am Ende lschen

open OUT, '>'. ($filename || 'test') . '.z80';
#open OUT, '>test.bin';
binmode OUT;

# Z1013-Headersave-Header
#KOPFAUFBAU: Byte 0-1 Anfangsadresse 
#               2-3   Endadresse
#               4-5   Startadresse
#               6-0BH frei fuer Zusatzinformationen
#                 0CH Typkennzeichen
#             0DH-0FH 3 x 0D3H = Kopfkenzeichen
#                     (An diesem D3 erkennt jedes Kopfsuchpro-
#                      gramm einen Kopf !!)
#             10H-1FH 16 Byte Namensblock.

#Header schreiben
             
my $header = pack("
		SSS		   # aadr, eadr, sadr: word;
		A6		   # frei fuer Zusatzinformationen
		a1                 # Typkennzeichen
		a3                 # Kopfkennzeichen 3x 0xD3
		A16                # dateiname
		"
	,$aadr, $eadr, $sadr, "AC1", "C", "\xD3"x3, $filename);

#Daten schreiben
syswrite OUT, $header; 
syswrite OUT, pack ("C*", @tapbuf); 
close OUT;

#########################################
# bin-Datei schreiben
#########################################

# bin-datei
$filename=sprintf"%s_%.4X_%.4X_%.4X.bin", $filename, $aadr, $eadr, $sadr;
open OUT, '>'. $filename;
binmode OUT;
#Daten schreiben
syswrite OUT, pack ("C*", @tapbuf); 
close OUT;


# folge noch ein File?
if (not eof(IN)) {goto neues_file};

close IN;

__END__

